#ifndef SCT_LIB_H
#define SCT_LIB_H

#include <stdint.h>
// The following macros are error codes.
#define E_LOCK_FILE 1001
#define E_OPEN_PLCY_FILE 1002
#define E_INVALID_ARGUMENT 1003
#define E_POLICY_NOEXIST 1004
#define E_POLICY_EXIST 1005
#define E_POLICY_FORMAT 1006
#define E_MEMORY_ALLOC_FAIL 1007
#define E_POLICY_RELOAD_FAIL 1008
#define E_USB_TRAVERSE_FAIL 1009
#define E_NETLINK_COMM_FAIL 1010
#define E_SCT_M_NOEXIST 1011
#define E_RENAME 1012
#define E_PERMISSION_DENIED 1013

struct usb_policy {
    uint32_t vid; /* usb dev vid */
    uint32_t pid; /* usb dev pid */
    uint32_t cls; /* usb dev class */
};

// interface type
enum intf_t {
    SERIAL_TYPE = 0x1,
    INTF_NUM
};

/**
 * @breif Add the interface policy for a given interface type from the configuration file.
 * @param intf_info (input): The type of interface whose policy is to be added
 *
 * @return int            Returns 0 on success, otherwise returns an error code
 *
 * ```
 * enum intf_t my_intf_type = SERIAL_TYPE;
 *
 * int result = sct_add_interface_policy(my_intf_type);
 * if (result == 0) {
 *     printf("Interface policy added successfully.\n");
 * } else {
 *     printf("Failed to add interface policy, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 *
 */
int sct_add_interface_policy(enum intf_t intf_info);

/**
 * @breif Delete the interface policy for a given interface type from the configuration file.
 * @param intf_info (input): The type of interface whose policy is to be deleted
 *
 * @return int            Returns 0 on success, otherwise returns an error code
 *
 * ```
 * enum intf_t my_intf_type = SERIAL_TYPE;
 *
 * int result = sct_del_interface_policy(my_intf_type);
 * if (result == 0) {
 *     printf("Interface policy deleted successfully.\n");
 * } else {
 *     printf("Failed to delete interface policy, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 *
 */
int sct_del_interface_policy(enum intf_t intf_info);

/**
 * @brief Retrieve interface type policy information
 * @param intf_info (output): A pointer to an array where the interface type information will be stored.
 * @param cnt (output): A pointer to an integer where the number of interface types will be stored.
 *
 * @return int            Returns 0 on success, otherwise returns an error code
 * ```
 * enum intf_t intf_info[INTF_NUM - 1];
 * uint32_t cnt = INTF_NUM - 1;
 *
 * int result = sct_get_interface_policy(intf_info, &cnt);
 * if (result == 0) {
 *     printf("Successfully retrieved %u interface policies.\n", cnt);
 *     // Process interface policies...
 * } else {
 *     printf("Failed to retrieve interface policies, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 */
int sct_get_interface_policy(enum intf_t *intf_info, uint32_t *cnt);

/**
 * @brief Add a USB policy from the policy configuration file
 * @param usb_info A pointer to a usb_policy structure containing the policy to add
 * @return int     Returns 0 on success, otherwise returns an error code
 *
 * Example usage:
 * ```
 * struct usb_policy my_usb_policy = {
 *  .vid = 1234,
 *  .pid = 5678,
 *  .cls = 9
 * };
 *
 * int result = sct_add_usb_policy(&my_usb_policy);
 * if (result == 0) {
 *  printf("USB policy added successfully.\n");
 * } else {
 *  printf("Failed to add USB policy, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 */
int sct_add_usb_policy(struct usb_policy *usb_info);

/**
 * @brief Delete a USB policy from the policy configuration file
 * @param usb_info A pointer to a usb_policy structure containing the policy to delete
 * @return int     Returns 0 on success, otherwise returns an error code
 *
 * Example usage:
 * ```
 * struct usb_policy my_usb_policy = {
 *  .vid = 1234,
 *  .pid = 5678,
 *  .cls = 9
 * };
 *
 * int result = sct_del_usb_policy(&my_usb_policy);
 * if (result == 0) {
 *  printf("USB policy deleted successfully.\n");
 * } else {
 *  printf("Failed to delete USB policy, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 */
int sct_del_usb_policy(struct usb_policy *usb_info);

/**
 * @brief Retrieve USB policy information
 *
 * @param usb_info_array  A pointer to an array of usb_policy structures to store the retrieved USB policy information (output parameter)
 * @param cnt A pointer to a uint32_t indicating the size of the usb_info_array array (input/output parameter)
 *
 * @return int Returns 0 on success, otherwise returns an error code
 *
 * Example usage:
 * ```
 * struct usb_policy policies[10];
 * uint32_t count = 10;
 * int result = sct_get_usb_policy(policies, &count);
 * if (result == 0) {
 *     // Successfully handle the retrieved USB policy information
 * } else {
 *     // Handle error situation
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 */

int sct_get_usb_policy(struct usb_policy *usb_info_array, uint32_t *cnt);
/**
 * @brief loads the policy file into the kernel.
 *
 * This function does not take any parameters.
 * @return int Returns 0 on success, otherwise returns an error code
 *
 * Example usage:
 * ```
 * int result = sct_reload();
 * if (result == 0) {
 *     printf("Policy file successfully loaded into kernel.\n");
 * } else {
 *     printf("Failed to load policy file into kernel, error code: %d\n", result);
 * }
 * ```
 * Thread Safety:
 * - This function is not thread-safe. Additional synchronization mechanisms are required to protect shared
 *   resources in a multi-threaded environment.
 */
int sct_reload(void);
#endif
